﻿#include <algorithm>
#include <stdlib.h>
#include <unordered_set>


#include "MeshLoader.h"

std::vector<Node>& MeshLoader::getNodes() 
{
	return nodes;
}

std::vector<FiniteElement>& MeshLoader::getFiniteElements() 
{
	return elements;
}

std::vector<BoundaryFiniteElement>& MeshLoader::getBoundaryFiniteElements() 
{
	return surfaces;
}



std::vector<FiniteElement> MeshLoader::getElementsByID(const int id1, const int id2, const int id3) 
{
	std::vector<FiniteElement> result;
	auto current_iter = elements.begin();
	auto last_iter = elements.end();
	while (current_iter != last_iter) 
	{
		current_iter = std::find_if(current_iter, last_iter, [id1, id2, id3](FiniteElement current_element) {
			return std::find(current_element.nodesID.begin(), current_element.nodesID.end(), id1) != current_element.nodesID.end()
				&& std::find(current_element.nodesID.begin(), current_element.nodesID.end(), id2) != current_element.nodesID.end()
				&& std::find(current_element.nodesID.begin(), current_element.nodesID.end(), id3) != current_element.nodesID.end(); });
		if (current_iter != last_iter) 
		{
			result.push_back(*current_iter);
			++current_iter;
		}
	}
	return result;
}

std::vector<FiniteElement> MeshLoader::getElementsByEdge(const int id1, const int id2) 
{
	std::vector<FiniteElement> result;
	auto current_iter = elements.begin();
	auto last_iter = elements.end();
	while (current_iter != last_iter) 
	{
		current_iter = std::find_if(current_iter, last_iter, [id1, id2](FiniteElement current_element) {
			return std::find(current_element.nodesID.begin(), current_element.nodesID.end(), id1) != current_element.nodesID.end()
				&& std::find(current_element.nodesID.begin(), current_element.nodesID.end(), id2) != current_element.nodesID.end(); });
		if (current_iter != last_iter) 
		{
			result.push_back(*current_iter);
			++current_iter;
		}
	}
	return result;
}

std::vector<FiniteElement> MeshLoader::getElementsByOneID(const int id1) 
{
	std::vector<FiniteElement> result;
	auto current_iter = elements.begin();
	auto last_iter = elements.end();
	while (current_iter != last_iter) 
	{
		current_iter = std::find_if(current_iter, last_iter, [id1](FiniteElement current_element) {
			return std::find(current_element.nodesID.begin(), current_element.nodesID.end(), id1) != current_element.nodesID.end(); });
		if (current_iter != last_iter) 
		{
			result.push_back(*current_iter);
			++current_iter;
		}
	}
	return result;
}

std::vector<Node> MeshLoader::getNodesByBoundary(int boundaryID) 
{
	std::set<Node> result;
	auto current_iter = surfaces.begin();
	auto last_iter = surfaces.end();
	while (current_iter != last_iter) 
	{
		current_iter = std::find_if(current_iter, last_iter, [boundaryID](BoundaryFiniteElement current_element) {
			return current_element.boundaryID == boundaryID; });
		if (current_iter != last_iter) 
		{
			for (const auto& it : current_iter->nodesID) 
			{
				auto node_by_ID = std::find_if(nodes.begin(), nodes.end(), [it](Node n) { return n.nodeID == it; });
				result.insert(*node_by_ID);
			}
			++current_iter;
		}
	}
	return std::vector<Node>(result.begin(), result.end());
}

std::vector<FiniteElement> MeshLoader::getElementsByBoundary(int boundaryID) 
{
	std::set<FiniteElement> result;
	auto current_iter = surfaces.begin();
	auto last_iter = surfaces.end();
	while (current_iter != last_iter) 
	{
		current_iter = std::find_if(current_iter, last_iter, [boundaryID](BoundaryFiniteElement current_element) {
			return current_element.boundaryID == boundaryID; });
		if (current_iter != last_iter) 
		{
			auto element_by_ID = std::find_if(elements.begin(), elements.end(), [current_iter](FiniteElement n) {
				return n.elementID == current_iter->elementID; });
			result.insert(*element_by_ID);
			++current_iter;
		}
	}
	return std::vector<FiniteElement>(result.begin(), result.end());
}

std::vector<BoundaryFiniteElement> MeshLoader::getsSurfacesByBoundary(int boundaryID) 
{
	std::vector<BoundaryFiniteElement> result;
	auto current_iter = surfaces.begin();
	auto last_iter = surfaces.end();
	while (current_iter != last_iter) 
	{
		current_iter = std::find_if(current_iter, last_iter, [boundaryID](BoundaryFiniteElement current_element) {
			return current_element.boundaryID == boundaryID; });
		if (current_iter != last_iter) 
		{
			result.push_back(*current_iter);
			++current_iter;
		}
	}
	return result;
}

void MeshLoader::insertCenterNode() {
	std::unordered_set<Edge, std::hash<Edge>> first;
	auto edges=sortEdges();
	for (auto& it : elements) 
    {
		std::vector<int> current_nodes(it.nodesID);
		for (auto i = current_nodes.begin(); i < current_nodes.end() - 1; ++i)
			for (auto j = i + 1; j < current_nodes.end(); ++j) 
			{
				Edge current_edge(*i, *j, 0);
				auto founded_edge = std::find(edges.cbegin(), edges.cend(), current_edge);
				if (founded_edge == edges.cend()) {
					Node left_node = *getNodeByID(*i);
					Node right_node = *getNodeByID(*j);
					int center_id_ = getLastNodeID() + 1;
					Node center_node(center_id_, (left_node.x + right_node.x) / 2,
						(left_node.y + right_node.y) / 2, (left_node.z + right_node.z) / 2, false);

					current_edge.updateCenter(center_id_);
					nodes.push_back(center_node);
					edges.insert(current_edge);
					it.nodesID.push_back(center_id_);
				}
			}
		it.nodesID.shrink_to_fit();
	}

	for (auto& it : surfaces) 
	{
		std::vector<int> current_nodes(it.nodesID);
		for (auto i = current_nodes.begin(); i < current_nodes.end() - 1; ++i)
			for (auto j = i + 1; j < current_nodes.end(); ++j) 
			{
				Edge current_edge(*i, *j, 0);
				auto founded_edge = std::find(edges.cbegin(), edges.cend(), current_edge);
				if (founded_edge != edges.cend())
					it.nodesID.push_back(founded_edge->center_id);
			}
		it.nodesID.shrink_to_fit();
	}

	surfaces.shrink_to_fit();
	nodes.shrink_to_fit();
}



Node* MeshLoader::getNodeByID(int id) 
{
	for (auto& cur : nodes)
		if (cur.nodeID == id)
			return &cur;
	return 0;
}

int MeshLoader::getLastNodeID() 
{
	return nodes.rbegin()->nodeID;
}



std::vector<int> MeshLoader::getNodeNeighbours(int p_nodeID) 
{
	std::set<int> result_in;
	for (const auto& i : getElementsByOneID(p_nodeID)) 
	{
		result_in.insert(i.nodesID.begin(), i.nodesID.end());
	}
	std::vector<int> result_out(result_in.begin(), result_in.end());
	auto it = std::find(result_out.begin(), result_out.end(), p_nodeID);
	result_out.erase(it);

	return result_out;
}


void MeshLoader::Print_Data() 
{
	std::cout << getNodes() << std::endl
		<< getFiniteElements() << std::endl
		<< getBoundaryFiniteElements() << std::endl;
}

std::unordered_set <Edge, std::hash<Edge>> MeshLoader::sortEdges()
{
	std::unordered_set <Edge, std::hash<Edge>> edges;
	std::unordered_set <Edge, std::hash<Edge>> result;
	auto current_iter = edges.begin();
	auto last_iter = edges.end();
	for (auto& i : edges)
	{
		for (auto it=++current_iter; it != edges.end(); (it->first_node_id == i.last_node_id && it->last_node_id == i.first_node_id) ? it = edges.erase(it) : ++it)
		{
			result.insert(*it);
		}
	}
	return result;
}
void MeshLoader::sortEdges(std::vector<Edge>& p_vector) {
	std::vector<Edge> copy_vec;
	std::copy(p_vector.begin(), p_vector.end(), copy_vec.begin());
	auto first_iter = p_vector.begin();
	auto last_iter = p_vector.end();
	auto last_iter2 = p_vector.rbegin();
	auto first_iter2 = p_vector.rend();

	

	for (auto i = first_iter; i != last_iter; ++i) {
		for (auto j = first_iter2; j != last_iter2; ++j) {
			p_vector.erase(remove_if(copy_vec.begin(), copy_vec.end(), 
				[i, j]() {return (i->first_node_id == j->last_node_id && i->last_node_id == j->first_node_id); }));
		}
	}
	
}